/**
  *  \file client/chart/keymapmode.cc
  *  \brief Starchart Mode: WKeymapChartMode
  *
  *  (c) 2001-2015 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/chart/keymapmode.h"
#include "client/chartloc.h"
#include "cpluslib/format.h"
#include "gfx/palette.h"
#include "int/simple.h"
#include "ui/event.h"
#include "ui/skin.h"
#include "ui/uiglobal.h"
#include "util/atom.h"
#include "util/nls.h"

namespace {
    /* FIXME: these are the same constants as in keymaphandler.cc */

    /* Pop up the keymap handler after two seconds. */
    const int POPUP_TICKS = 2 * TIMER_HZ;

    /* Font to use */
    const int FONT = FONT_NORMAL_BOLD;
}

WKeymapChartMode::WKeymapChartMode(IntKeymapRef km, int prefix)
    : WSimpleChartMode(ChartMode_Prefix),
      keymap(km),
      prefix(prefix),
      time(0)
{ }

WKeymapChartMode::~WKeymapChartMode()
{ }

void
WKeymapChartMode::drawOverlays(GfxCanvas& can, GChartLocation& loc)
{
    // Other overlays below our text
    WSimpleChartMode::drawOverlays(can, loc);

    if (time >= POPUP_TICKS) {
        string_t text = format(_("Keymap %s"), keymap->getName());
        int width     = fonts[FONT]->getTextWidth(text) + 6;
        int height    = fonts[FONT]->getTextHeight(text) + 2;
        GfxPoint pt   = loc.getViewport().getCenter();
        GfxRect r(pt.x - width/2, pt.y - height/2 + 20, width, height);

        GfxContext ctx;
        ctx.useCanvas(can);
        ctx.useColorScheme(GfxStandardColorScheme::instance);

        // FIXME: same color as for prefix arg
        drawSolidBar(ctx, r, COLOR_FIRE + 29);

        ctx.setColor(COLOR_DARKYELLOW);
        drawHLine(ctx, r.x, r.getBottomY()-1, r.getRightX()-1);
        drawVLine(ctx, r.getRightX()-1, r.y, r.getBottomY()-2);

        ctx.setColor(COLOR_FIRE + 30);
        drawHLine(ctx, r.x+1, r.y, r.getRightX()-1);
        drawVLine(ctx, r.x, r.y, r.getBottomY()-2);

        ctx.setColor(COLOR_BLACK);
        ctx.useFont(*fonts[FONT]);
        outText(ctx, r.x + 3, r.y + 1, text);
    }
}

bool
WKeymapChartMode::handleEvent(const UIEvent& event, GChartLocation& loc)
{
    if (event.type == ev_Key) {
        uint32_t command = (keymap->lookupCommand(event.key.code));
        if (command != 0) {
            // This key was bound. Use it.
            // First, copy the parameters to local variables so we can delete ourselves.
            int arg = prefix;
            delete this;
            executeStatement(atomStr(command), arg);
            return true;
        } else if (event.key.code == SDLK_ESCAPE) {
            // Escape
            delete this;
            return true;
        } else if (getClassOfKey(event.key.code & ~ss_AllMod) == KeyNormal) {
            // Not bound, but something that looks like a key (i.e. not a modifier).
            if (time < POPUP_TICKS) {
                time = POPUP_TICKS;
                loc.markChanged();
            }
            return true;
        } else {
            // Something else, e.g. Shift.
            return WSimpleChartMode::handleEvent(event, loc);
        }
    } else if (event.type == ev_Tick) {
        // Popup time?
        if (++time == POPUP_TICKS) {
            loc.markChanged();
        }
        return WSimpleChartMode::handleEvent(event, loc);
    } else {
        return false;
    }
}

void
WKeymapChartMode::startKeymapMode(WChartModeList& chart_modes, IntKeymapRef km, int prefix)
{
    chart_modes.pushMode(*new WKeymapChartMode(km, prefix));
}
