/**
  *  \file client/screens/basetaskscreen.cc
  *  \brief Starbase Task Screen
  *
  *  (c) 2001-2015 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/screens/basetaskscreen.h"
#include "client/screens/planettaskscreen.h"
#include "client/tasksupport.h"
#include "client/wscanchart.h"
#include "cpluslib/format.h"
#include "game/global.h"
#include "game/obj.h"
#include "game/planet.h"
#include "int/exec.h"
#include "int/if/planetif.h"
#include "int/intglobal.h"
#include "int/process.h"
#include "util/nls.h"

namespace {
    class WBaseAutoTaskScreen : public WAutoTaskScreen {
     public:
        WBaseAutoTaskScreen(WPlanetAutoTaskSelection& selection);
        void update();
        virtual void enumContexts(ptr_vector<IntContext>& ctx);
    };
}

/************************** WBaseAutoTaskScreen **************************/

WBaseAutoTaskScreen::WBaseAutoTaskScreen(WPlanetAutoTaskSelection& selection)
    : WAutoTaskScreen(selection, WScreenId::BaseTaskScreen)
{
    createTileLayout(*this, "BASETASKSCREEN", selection);
    addChart(holder.add(new WScannerChartWidget()));
    setKeymap(IntKeymap::getKeymapByName("BASETASKSCREEN"));
}

void
WBaseAutoTaskScreen::update()
{
    if (GMapObject* mo = dynamic_cast<GMapObject*>(getObjectSelection().getCurrentObject())) {
        WScannerChartWidget& chart = *getScanner().getChart();
        chart.setCenterPosition(mo->getPos());
        chart.setScannerOrigin(mo->getPos());
        chart.setScannerTarget(mo->getPos());
    }
}

void
WBaseAutoTaskScreen::enumContexts(ptr_vector<IntContext>& ctx)
{
    ctx.push_back(new IntPlanetContext(selection.getCurrentIndex()));
}


void
doBaseAutoTaskScreen()
{
    WPlanetAutoTaskSelection at(current_base_selection, IntExecutionContext::pkBaseTask);
    WBaseAutoTaskScreen screen(at);
    at.init();
    screen.runScreen();
    at.done();
}
