/**
  *  \file flak/vis-ui.h
  *
  *  (c) 2001-2015 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_FLAK_VIS_UI_H
#define PCC_V2_FLAK_VIS_UI_H

#include "gfx/sprite3d.h"
#include "flak/flak.h"
#include "flak/vis.h"
#include "ui/ui.h"
#include "ui/widgets/button.h"

class FlakPlayWidget;

class FlakTimeDisplay : public GfxSprite3D {
    FlakBattle& battle;
    uint32 max_time;
    int digits;
 public:
    FlakTimeDisplay(const GfxRect& player_size, FlakBattle& b, uint32 max_time);
    void tick();
    void draw(GfxCanvas& can);
};

class FlakDistanceDisplay : public GfxSprite3D {
    FlakPlayWidget& widget;
 public:
    FlakDistanceDisplay(const GfxRect& player_size, FlakPlayWidget& widget);
    void tick();
    void draw(GfxCanvas& can);
};

class FlakShipDisplay : public GfxSprite3D {
    FlakBattle&        battle;
    FlakBattle::ship_t ship;
    int width;
    void adjustSize();
 public:
    FlakShipDisplay(const GfxPoint pt, int width, FlakBattle& battle, FlakBattle::ship_t ship)
        : GfxSprite3D(GfxRect(pt.x, pt.y, 0, 0), 0, FlakFancyVisualizer::layer_Text),
          battle(battle), ship(ship), width(width)
        { adjustSize(); }
    void tick();
    void draw(GfxCanvas& can);
    void setShipNumber(FlakBattle::ship_t ship);

    FlakBattle::ship_t getShipNumber() const
        { return ship; }
};

class FlakSimpleDivider : public UIWidget {
 public:
    FlakSimpleDivider()
        : UIWidget()
        { }
    void drawContent(GfxCanvas& can);
    bool handleEvent(const UIEvent& e, bool second_pass);
    void getLayoutInfo(LayoutInfo& li);
};

class FlakShipList : public UIWidget {
    enum {
        FONT = FONT_NORMAL
    };
    struct Item {
        enum What {
            fFleet,
            fShip
        };
        unsigned  index   : 12;
        What      what    : 3;
        bool      visible : 1;

        int getHeight() const;

        Item(unsigned index, What what, bool visible)
            : index(index), what(what), visible(visible)
            { }
    };
    friend struct Item;
    FlakBattle& battle;
    std::vector<Item> content;
    unsigned current;
    unsigned top;
    unsigned num_dead_units;

    void toggleFleetVisibility();
public:
    FlakShipList(FlakBattle& battle);
    void drawContent(GfxCanvas& can);
    bool handleEvent(const UIEvent& e, bool second_pass);
    void showShip(FlakBattle::ship_t ship);
    void setFleetStatus(FlakBattle::fleet_t fleet, bool status);
    void setFleetStatusByPosition(unsigned index, bool status);
    void scrollBy(int diff);
    void adjustTop();
    void getLayoutInfo(LayoutInfo& li);

    void onUpdate();

    FlakBattle::ship_t getSelectedShip() const;

    Signal0 sig_moved;
    Signal1<FlakBattle::ship_t> sig_ship_clicked;
};

class FlakPlayControlWidget : public UIGroup {
    UIGridLayout grid;
 public:
    UIButton rewind_start;
    UIButton rewind;
    UIButton play;
    UIButton forward;
    UIButton forward_end;

    FlakPlayControlWidget();
    ~FlakPlayControlWidget();
};

class FlakMainControlWidget : public UIGroup {
    UISpacer spacer;
 public:
    UIButton exit_button;

    FlakMainControlWidget();
    ~FlakMainControlWidget();
    void drawContent(GfxCanvas& can);
};

class FlakPlayWidget : public UIWidget {
    FlakVcrEntry&        entry;
    FlakFancyVisualizer* vis;
    bool                 playing : 1;
    bool                 mouse_down : 1;

    Ptr<FlakShipDisplay> ship_displays[2];

    FlakShipList&        ship_list;

    void createVisualizer(bool rewind);
 public:
    FlakPlayWidget(FlakVcrEntry& entry, FlakShipList& list);
    ~FlakPlayWidget();
    bool handleEvent(const UIEvent& e, bool second_pass);
    void drawContent(GfxCanvas& can);
    FlakBattle& getBattle()
        { return entry.getBattle(); }
    int32 getTotalTime() const
        { return entry.getTotalTime(); }
    void updateSprites();
    void setPlayStatus(bool playing);
    void getLayoutInfo(LayoutInfo& li);
    void onResize(GfxRect extent, bool by_layout);
    void setShipTrack(int side, FlakBattle::ship_t ship);

    void setTime(int32 time);
    void onRewindToStart();
    void onRewind();
    void onPlay();
    void onForward();
    void onForwardToEnd();
    void onShipClicked(FlakBattle::ship_t ship);

    FlakBattle::ship_t getShipTrack(int side) const
        { return ship_displays[side] ? ship_displays[side]->getShipNumber() : FlakBattle::no_ship; }

    Signal0 sig_update;
    Signal0 sig_play_status;
};

#endif
