/**
  *  \file int/if/hullif.h
  *  \brief Interpreter Game Interface: Hull Properties
  *
  *  (c) 2001-2015 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_INT_IF_HULLIF_H
#define PCC_V2_INT_IF_HULLIF_H

#include "int/value.h"
#include "int/context.h"

class GHull;
class IntArgBlock;

IntValue*   IFHullGet(IntArgBlock& args);
IntContext* IFHullMake();
int32_t     IFHullDim(int32_t dim);

/** Hull context. */
class IntHullContext : public IntContext {
    int hid;
 public:
    IntHullContext(int hid);
    ~IntHullContext();

    // IntContext:
    int32_t   lookup(const string_t& name);
    void      set(int32_t index, IntValue* value);
    IntValue* get(int32_t index);
    bool      next();
    IntContext* clone();
    GObject*  getObject();
    string_t  toString(bool readable);
    void      store(IntTagNode& sv, Stream& aux);
    void      enumProperties(IntPropertyAcceptor& acceptor);
};

/** Definition of hull properties. */
enum IntHullProperty {
    ihpMaxBeams,
    ihpMaxCargo,
    ihpMaxFuel,
    ihpMaxCrew,
    ihpNumEngines,
    ihpNumFighterBays,
    ihpSpecial,
    ihpImage,
    ihpImage2,
    ihpMaxTorpLaunchers
};

IntValue* getHullProperty(const GHull& h, IntHullProperty isp);
void      setHullProperty(GHull& h, IntHullProperty isp, IntValue* value);

#endif
