/**
  *  \file int/subr.h
  *  \brief Interpreter: Subroutine Value
  *
  *  (c) 2001-2015 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_INT_SUBR_H
#define PCC_V2_INT_SUBR_H

#include "int/compound.h"
#include "int/bytecode.h"

/** Subroutine value. Value referring to a subroutine. This is used to
    implement the 'pushsub' opcode. */
class IntSubroutineValue : public IntCallableValue {
    IntBCORef bco;
 public:
    IntSubroutineValue(IntBCORef bco);
    ~IntSubroutineValue();
    IntBCORef getBytecodeObject();

    // IntCallableValue:
    virtual void call(IntExecutionContext& exc, IntDataSegment& args, bool want_result);
    virtual bool isProcedureCall();
    virtual int32_t getDimension(int32_t which);
    virtual IntContext* makeFirstContext();

    // IntValue:
    virtual string_t toString(bool readable);
    virtual void store(IntTagNode& tag, Stream& aux);
    virtual IntSubroutineValue* clone();
};

#endif
